package com.javacreed.examples.axon;

import org.axonframework.commandhandling.CommandCallback;
import org.axonframework.commandhandling.CommandMessage;
import org.axonframework.commandhandling.gateway.CommandGateway;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.CommandLineRunner;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.context.annotation.Bean;

import com.javacreed.examples.axon.task.AddNoteToTaskCommand;
import com.javacreed.examples.axon.task.CompleteTaskCommand;
import com.javacreed.examples.axon.task.CreateTaskCommand;
import com.javacreed.examples.axon.task.Task.AddNoteResult;
import com.javacreed.examples.axon.task.TaskId;
import com.javacreed.examples.axon.task.TaskName;
import com.javacreed.examples.axon.task.TaskNote;

@SpringBootApplication
public class Demo {

  private static final Logger LOGGER = LoggerFactory.getLogger(Demo.class);

  public static void main(final String[] args) {
    SpringApplication.run(Demo.class, args);
  }

  @Bean
  CommandLineRunner runner(final CommandGateway gateway) {
    return args -> {

      /* Create the task */
      final TaskId id = TaskId.random();
      gateway.send(CreateTaskCommand.of(id, TaskName.of("Test")));

      /* Used to simply document the outcome of the add note command */
      final CommandCallback<AddNoteToTaskCommand, AddNoteResult> callback = new CommandCallback<AddNoteToTaskCommand, AddNoteResult>() {
        @Override
        public void onFailure(final CommandMessage<? extends AddNoteToTaskCommand> commandMessage,
            final Throwable cause) {}

        @Override
        public void onSuccess(final CommandMessage<? extends AddNoteToTaskCommand> commandMessage,
            final AddNoteResult result) {
          Demo.LOGGER.debug("Result: {}", result);
        }
      };

      /* Add a note */
      gateway.send(AddNoteToTaskCommand.of(id, TaskNote.of("Note 1")), callback);

      /* Complete the task (no more notes can be added now) */
      gateway.send(CompleteTaskCommand.of(id));

      /* Add the second note after the task is closed. The note will be rejected */
      gateway.send(AddNoteToTaskCommand.of(id, TaskNote.of("Note 2")), callback);

      Demo.LOGGER.debug("Done");
    };
  }
}
