package com.javacreed.examples.axon.task;

import java.util.LinkedHashSet;
import java.util.Set;

import org.axonframework.commandhandling.CommandHandler;
import org.axonframework.commandhandling.model.AggregateIdentifier;
import org.axonframework.commandhandling.model.AggregateLifecycle;
import org.axonframework.eventsourcing.EventSourcingHandler;
import org.axonframework.spring.stereotype.Aggregate;

@Aggregate
public class Task {

  public static enum AddNoteResult {
    ADDED, ALREADY_COMPLETE, ALREADY_EXISTS;
  }

  @AggregateIdentifier
  private TaskId id;
  private TaskStage stage;

  private Set<TaskNote> notes;

  @SuppressWarnings("unused")
  private Task() {}

  @CommandHandler
  public Task(final CreateTaskCommand command) {
    AggregateLifecycle.apply(TaskCreatedEvent.of(command));
  }

  @EventSourcingHandler
  public void apply(final TaskCompletedEvent event) {
    stage = TaskStage.COMPLETE;
  }

  @EventSourcingHandler
  public void apply(final TaskCreatedEvent event) {
    id = event.getId();
    stage = TaskStage.OPEN;
    notes = new LinkedHashSet<>();
  }

  @CommandHandler
  public AddNoteResult handle(final AddNoteToTaskCommand command) {
    if (stage == TaskStage.COMPLETE) {
      AggregateLifecycle.apply(TaskAlreadyCompletedEvent.of(command));
      return AddNoteResult.ALREADY_COMPLETE;
    }

    if (notes.contains(command.getNote())) {
      AggregateLifecycle.apply(NoteAlreadyExistsInTaskEvent.of(command));
      return AddNoteResult.ALREADY_EXISTS;
    }

    AggregateLifecycle.apply(NoteAddedToTaskEvent.of(command));
    return AddNoteResult.ADDED;
  }

  @CommandHandler
  public void handle(final CompleteTaskCommand command) {
    if (stage == TaskStage.OPEN) {
      AggregateLifecycle.apply(TaskCompletedEvent.of(command));
    }
  }
}
